/*
 * Copyright 1992 the Board of Trustees of the Leland Stanford Junior
 * University. Official permission to use this software is included in
 * the documentation. It authorizes you to use this file for any
 * non-commercial purpose, provided that this copyright notice is not
 * removed and that any modifications made to this file are commented
 * and dated in the style of the example below.
 */

/*
 *
 *  source file:   ./xtpanel/button.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), November 20 1992 
 * 	added toggle object
 * Steve Cole (SEP) January 14 1993
 *      Added toggle_sync routine.
 * Steve Cole (SEP) May 7 1993
 *	Fixed the name defaulting for toggles.
 * Steve Cole (SEP) May 26 1993
 *      Toggle value is now set properly if initial state is True.
 * Steve Cole (SEP) July 23 1993
 *      Added onvalue and offvalue for toggles. Revised the way
 *	assignment of toggle values works.
 * Steve Cole (SEP) September 19 1993
 *      Made onvalue and offvalue settable.
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>

#include <stdio.h>

#include "object.h" 
#include "tree.h"
#include "builders.h"

typedef struct buttoninfo {
    char* type;
    char* onvalue;
    char* offvalue;
} _binf;

void build_button(root,parent,type)
     entry *root;
     Objdef* parent;
     char *type;
{
    Objdef *object;
    char* label;
    struct buttoninfo *button_info;
    char defname[12];
    Widget button;
    Arg args[20];
    Boolean state;
    int narg;
    static int numbutt=1, numtog=1;
    extern void button_callback();
    extern void button_update();
    extern void toggle_sync();
    
    /* create new object */
    object = new_object(parent);

    /* construct default button name */
    if (!strcmp(type,"button"))
      sprintf(defname,"button%d",numbutt++);
    else
      sprintf(defname,"toggle%d",numtog++);
    
    /* find label, name, action in tree */
    object->name = get_value(root,"name",defname);
    label = get_value(root,"label",object->name);
    object->action = parse_actions(object->name,root);
    button_info = (struct buttoninfo*) XtMalloc( sizeof( struct buttoninfo ) );
    /* save type (button, toggle) for use by callback routines */
    button_info->type = XtNewString(type);
    /* set values */
    if (!strcmp(button_info->type,"button")) {
    	object->value = get_value(root,"value",label);
    } else {
    	button_info->onvalue = get_value(root,"onvalue",label);
    	button_info->offvalue = get_value(root,"offvalue","");
	/* toggles are off initially unless value matches onvalue */
    	object->value = get_value(root,"value","");
    }

    narg = 0;
    XtSetArg(args[narg], XtNlabel, label); narg++;

    /* common parameters */
    common_tags(parent->widgetname,root,args,&narg,SET_BEFORE);

    if (!strcmp(type,"button")) 
    {
    	button = XtCreateManagedWidget(object->name,commandWidgetClass,
				   parent->widgetname,args,narg);
    } else {
    	button = XtCreateManagedWidget(object->name,toggleWidgetClass,
				   parent->widgetname,args,narg);
    }

    object->widgetname = button;
    object->info = button_info;
    object->updater = button_update;
    if (!strcmp(button_info->type,"toggle")) object->sync = toggle_sync;

    /* now hide object if needed */
    narg = 0;
    common_tags(button,root,args,&narg,SET_AFTER);

    /* for toggles, set object value to onvalue if initial state is on 
     * or if value=onvalue */
    if (!strcmp(button_info->type,"toggle"))
    {
        XtSetArg( args[0], XtNstate, &state );
        XtGetValues( object->widgetname, args, ONE );
        XtFree( object->value );
        if (state == TRUE || 
	  !strcmp(object->value,button_info->onvalue)) {
            object->value = XtNewString(button_info->onvalue);
            XtSetArg( args[0], XtNstate, TRUE );
            XtSetValues( object->widgetname, args, ONE );
        } else {
            object->value = XtNewString(button_info->offvalue);
            XtSetArg( args[0], XtNstate, FALSE );
            XtSetValues( object->widgetname, args, ONE );
        }
    } 
    
    XtAddCallback( button, XtNcallback, button_callback, (XtPointer) object);

}

/* callback used for buttons */
void
  button_callback(widget, client_data, callData)
Widget widget;
XtPointer client_data, callData;
{
    Objdef *object;
    struct buttoninfo *button_info;
    Arg arg[1];
    Boolean state;
    
    object = (Objdef *) client_data;
    button_info = (struct buttoninfo *) object->info;

    /* set toggle value if it is on */
    if (!strcmp(button_info->type,"toggle"))
    {
        XtSetArg( arg[0], XtNstate, &state );
        XtGetValues( object->widgetname, arg, ONE );
        if (state == TRUE ) 
    	    object->value = XtNewString(button_info->onvalue);
        else
	    object->value = XtNewString(button_info->offvalue);
    }

    perform_actions(object->name,object->action,1);
}

void button_update(object,value)
Objdef* object;
char* value;
{
    struct buttoninfo *button_info;
    Arg arg[1];
    Boolean state;

    button_info = (struct buttoninfo *) object->info;

    /*
     * if value matches toggle's on or off value,
     * change state. Otherwise change the on or off value
     * to the new one. 
     */
    /* set toggle value if it is on */
    if (!strcmp(button_info->type,"toggle"))
    {
	if (!strcmp(value,button_info->onvalue)) {
          object->value = XtNewString(button_info->onvalue);
          XtSetArg( arg[0], XtNstate, TRUE );
          XtSetValues( object->widgetname, arg, ONE );
	} else if (!strcmp(value,button_info->offvalue)) {
          object->value = XtNewString(button_info->offvalue);
          XtSetArg( arg[0], XtNstate, FALSE );
          XtSetValues( object->widgetname, arg, ONE );
	} else {
          XtSetArg( arg[0], XtNstate, &state );
          XtGetValues( object->widgetname, arg, ONE );
          if (state == TRUE) {
            button_info->onvalue = XtNewString(value);
            object->value = XtNewString(button_info->onvalue);
          } else {
            button_info->offvalue = XtNewString(value);
            object->value = XtNewString(button_info->offvalue);
          }
	}
    }
}

/*
 * this routine is called when a toggle's state is changed
 * by the SET action, to be sure that the state and the 
 * object value are in agreement.
 */
void toggle_sync(object,tag,value)
Objdef* object;
char* tag;
char* value;
{
    struct buttoninfo *button_info;
    Arg args[20];
    Boolean state;

    button_info = (struct buttoninfo *) object->info;

    /* handle onvalue and offvalue here */
    if (!strcmp(tag,"onvalue")) {
       button_info->onvalue = XtNewString(value);
    } else if (!strcmp(tag,"onvalue")) {
       button_info->offvalue = XtNewString(value);
    /* everything else goes to update_common_tag */
    } else {
	update_common_tag(object,tag,value);
    }

    /* set toggle value if it is on */
    if (!strcmp(button_info->type,"toggle"))
    {
        XtSetArg( args[0], XtNstate, &state );
        XtGetValues( object->widgetname, args, ONE );
	XtFree( object->value );
        if (state == TRUE ) 
            object->value = XtNewString(button_info->onvalue);
	else
	    object->value = XtNewString(button_info->offvalue);
    }
}
