/*
 * Copyright 1992 the Board of Trustees of the Leland Stanford Junior
 * University. Official permission to use this software is included in
 * the documentation. It authorizes you to use this file for any
 * non-commercial purpose, provided that this copyright notice is not
 * removed and that any modifications made to this file are commented
 * and dated in the style of the example below.
 */

/*
 *
 *  source file:   ./xtpanel/main.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), February 23 1993 
 * 1) added allow_resize to toplevel to let xtpanel resize itself.
 * 2) added support for form widget.
 * 3) improved fallback resources.
 * Steve Cole (SEP), February 24 1993 
 *      Added sensitive to list of recognized arguments.
 * Dave Nichols (SEP), April 29 1993
 *      Added return code to quit_xtpanel
 * Steve Cole (SEP), May 9 1993 
 *	Removed bad Scrollbar*orientation entry in fallback
 *      resources. Caused text scrollbars to be unusable when
 *      app-defaults were not found.
 * Steve Cole (SEP), June 7 1993 
 *      Added support for window object.
 * Dave Nichols (SEP), June 20 1993 
 *      Moved recursive calls to process_tree into composite object builder
 *	routines.
 *	Removed syntax checking in process_tree (now done at parse time).
 * Dave Nichols (SEP), Nov 29 1993
 * 	Added patch from  grant@cscc.oh.on.ca (Grant McDorman) to
 *      handle arrow scrollbar.
 */

#include "patchlevel.h"

#include <stdio.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xmu/Converters.h>

#include <X11/Xos.h>

#include "tree.h"
#include "object.h"
#include "builders.h"

String fallback_resources[] = {
    "*input:                  			True",
    "*Paned*resizable:   			True",
    "*Dialog*Command.label:   			ok",
    "*sliderbox*Command.label:			ok",
    "*Dialog*resizable:       			True",
    "*Dialog*value.translations: #override <Key>Return: return_key()",
    "*Box*allowResize:        			True",
    "*Box*resizable:        			True",
    "*SimpleMenu*leftMargin:	         	30",
    "*noborder.borderWidth:          		0",
    "*squeezebox*vSpace:                        0",
    "*squeezebox*hSpace:                        0",
    "*squeezebox*defaultDistance:               0",
    "*Form*top:                      		ChainTop",
    "*Form*bottom:                  		ChainTop",
    "*Form*left:                     		ChainLeft",
    "*Form*right:                    		ChainLeft",
#ifdef XAW_ARROW_SCROLLBAR
    "*graphbox*Scrollbar.translations:#override\\n\
      Meta<EnterNotify>: MoveThumb() NotifyThumb()\\n\
      Meta<LeaveNotify>: NotifyScroll(Proportional) EndScroll()\\n\
      Meta<Motion>: MoveThumb() NotifyThumb()",
#else
    "*graphbox*Scrollbar.translations:#override\\n\
      Meta<EnterNotify>: StartScroll(Continuous) MoveThumb() NotifyThumb()\\n\
      Meta<LeaveNotify>: NotifyScroll(Proportional) EndScroll()\\n\
      Meta<Motion>: MoveThumb() NotifyThumb()",
#endif
    "*togglebox*Toggle.translations:#override\\n\
      Meta<EnterNotify>:                set()\\n\
      Ctrl<EnterNotify>:                unset()",
    NULL,
};

extern void return_key_callback(),dialog_update_callback();
extern void text_update_callback(),graph_update_callback();
extern void field_return_key_callback(),grid_update_callback();
extern void quit_callback();
extern void tab_next_callback();

static XtActionsRec window_actions[] = {
    {"return_key",       return_key_callback},
    {"field_return_key", field_return_key_callback},
    {"update_dialog",    dialog_update_callback},
    {"update_text",      text_update_callback},
    {"tab_next",         tab_next_callback},
    {"update_graph",     graph_update_callback},
    {"update_grid",      grid_update_callback},
    {"quit",      	 quit_callback},
};

int		quitflag=0;

Widget toplevel;
XtAppContext app_con;
char* progname;
int stdin_input=0;
int stdin_used=0;

int io_err_handler( disp ) Display * disp; { exit( 0 ); }
extern void quit_xtpanel();
extern void parse_args(),add_pixmap_converter();
extern void process_tree();
    
int main(argc,argv)
     
     int argc;
     char **argv;
{
    entry *root;
    Arg args[20];
    int narg;
    Objdef* rootobj;
    
    /* set the io error handler */
    XSetIOErrorHandler( io_err_handler );

    /* this allows the xtpanel window to grow if needed */
    narg = 0;
    XtSetArg(args[narg], XtNallowShellResize, TRUE ); narg++;

    toplevel = XtAppInitialize(&app_con, "XTpanel", NULL, ZERO,
			       &argc, argv, fallback_resources,
			       args, narg);

    rootobj = new_object(0); 
    rootobj->widgetname = toplevel; 
    rootobj->name = XtNewString("toplevel"); 

    XtAddCallback( toplevel, XtNdestroyCallback, quit_callback, NULL);
    
    XtAppAddActions(app_con, window_actions, XtNumber(window_actions));

/*    add_pixmap_converter();*/
    
    /* make the root entry to the tree, it is called "root" */
    root= new_entry( "root", 4);
    
    /* now go and parse what is left of the command line */
    progname = XtNewString( *argv );
    argv++; argc--;
    
    parse_args( argc, argv, root );
    
    /* parse stdin if it pointing at something unless stdin_input is set */
    if( (!isatty( fileno(stdin) ))  && (!stdin_input) ) {
        parse_file( stdin, root );
	stdin_used=1;
    }
    
    /* print tree for debugging */
/*    print_entry( stderr,root,0 );*/
   

    /* create top level window  and all objects in it */
    build_window( root, rootobj, 0 );

    XtRealizeWidget(toplevel);
    
    /* do the action for all objects with action type string */
    /* so that strings are set to something when they are needed */
/*    all_actions();*/
    
    /* make panel just big enough to encompass all items */
    /* pass control to notifier */
    XtAppMainLoop(app_con);

    return 0;
}


/* 
  This routine take the fully parsed object tree and makes the
  appropriate widgets from each object.
  */

/* this routine may be called recursively (if the user defined sub boxes ) */

void process_tree( root,parent )
     entry *root;
     Objdef *parent;
{
    entry* curr;
      
    /* loop over first level of tree */
    for( curr=root->child; curr != (entry*)0; curr=curr->next ){
	
	/* here is the big switch on item type */
	
	if( !strcmp( curr->tag, "window" ) ){
	    /* create the new window  and all objects in it */
	    build_window(curr, parent, 1);

	}else if( !strcmp( curr->tag, "box" ) || 
            !strcmp( curr->tag, "vbox") ||
            !strcmp( curr->tag, "hbox") ||
            !strcmp( curr->tag, "pane") ||
            !strcmp( curr->tag, "vpane") ||
            !strcmp( curr->tag, "hpane")){
	    /* create the new box and all objects in it */
	     build_box(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "form" ) ){
	    /* create the new form and all objects in it */
	    build_form(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "message" ) ){
	    build_message(curr,parent);
	    
	}else if( !strcmp( curr->tag, "button" ) ||
		 !strcmp( curr->tag, "toggle" )){
	    build_button(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "choice" ) ){
	    build_choice(curr,parent);
	    
	}else if( !strcmp( curr->tag, "slider" ) ||
		 !strcmp( curr->tag, "scrollbar" )){
	    build_slider(curr,parent,curr->tag);

	}else if( !strcmp( curr->tag, "graph" ) ){
	    build_graph(curr,parent);
	    
	}else if( !strcmp( curr->tag, "grid" ) ||
		 !strcmp( curr->tag, "togglegrid" )){
	    build_grid(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "menubutton" ) ){
	    build_menubutton(curr,parent);
	    
	}else if( !strcmp( curr->tag, "list" ) ){
	    build_list(curr,parent);
	    
	}else if( !strcmp( curr->tag, "dialog" ) ){
	    build_dialog(curr,parent);
	    
	}else if( !strcmp( curr->tag, "text" ) || 
	          !strcmp( curr->tag, "field" ) ){
	    build_text(curr,parent,curr->tag);
	    	    
	}else if( !strcmp( curr->tag, "var" ) ||
	          !strcmp( curr->tag, "startup" ) ){
	    build_variable(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "timer" ) ){
	    build_timer(curr,parent);
	
	}else if( !strcmp( curr->tag, "input" ) ||
		  !strcmp( curr->tag, "output" ) ||
		  !strcmp( curr->tag, "pipe" ) ){
	    build_input(curr,parent,curr->tag);
	
	}else if( !strcmp( curr->tag, "template" ) ){
	    save_template(curr);
	
	}else if( is_template( curr->tag )){
	    build_template(curr,parent, curr->tag );
	
        }    
	
    } /* end of loop over objects in tree */
    
}

/*
 * Function name: quit_xtpanel
 * Description: quits by destroying toplevel widget.
 * Arguments: none
 * Returns: none
 */
extern Widget toplevel;
void quit_callback() { quit_xtpanel( 0 ); }


void quit_xtpanel( ret )
int ret;
{
    XtDestroyWidget((Widget)toplevel);
    exit(ret);
}

void flush_display()
{
    XFlush( XtDisplay( toplevel ) );
}

