/*
 * Copyright 1992 the Board of Trustees of the Leland Stanford Junior
 * University. Official permission to use this software is included in
 * the documentation. It authorizes you to use this file for any
 * non-commercial purpose, provided that this copyright notice is not
 * removed and that any modifications made to this file are commented
 * and dated in the style of the example below.
 */

/*
 *
 *  source file:   ./xtpanel/menubutton.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), 21 May 1993
 *      Button widget name now saved in object structure.
 * Dave Nichols (SEP), June 8 1993
 *      Don't re-expand items that come from a list.
 * Steve Cole (SEP), October 8 1993
 *      Added default separator.
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/bitmaps/dot>

#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>

#include <X11/Xaw/Cardinals.h>

#include "object.h" 
#include "item.h"
#include "tree.h"
#include "builders.h"

extern Widget toplevel;
static Pixmap mark;

void build_menubutton(root,parent)
     entry *root;
     Objdef* parent;
{
    Objdef *object;
    entry *curr;
    char *label, *value , *fromlist ;
    struct itemizedinfo *menu_info;
    struct itemdef *item;
    Arg args[20];
    int narg;
    Widget button, menu, menuitem;
    char defname[14];
    static int nummenu=1;
    int nitem=0;
    extern void menubutton_callback();
    extern void menubutton_update();
    
    /* create new object */
    object = new_object(parent);

    /* construct default button name */
    sprintf(defname,"menubutton%d",nummenu++);
    
    /* find label, name, action in tree */
    object->name = get_value(root,"name",defname);
    label = get_value(root,"label",object->name);
    object->action = parse_actions(object->name,root);
    menu_info = (struct itemizedinfo*) 
             XtMalloc( sizeof( struct itemizedinfo ) );
    menu_info->firstitem = (struct itemdef*) 0;
    /* define a default separator */
    menu_info->separator = XtNewString(" \n");
    
    object->info = menu_info;
    object->updater = menubutton_update;
    
    /* create the button and the menu */
    narg = 0;
    XtSetArg(args[narg], XtNlabel, label); narg++;
    /* common parameters */
    common_tags(parent->widgetname,root,args,&narg,SET_BEFORE);

    button = XtCreateManagedWidget(object->name,menuButtonWidgetClass,
                 parent->widgetname,args,narg);
    menu = XtCreatePopupShell("menu",simpleMenuWidgetClass,button,NULL,ZERO);
    
    object->widgetname = button;
    object->childwidget = menu;

    /* now hide object if needed */
    narg = 0;
    common_tags(button,root,args,&narg,SET_AFTER);

    /* first parse any itemlist entries into items */
    parse_itemlist( root, &(menu_info->separator) );
    
    /* loop over again to get items */
    for (curr = get_next(root,"item",(entry*) 0), nitem=0; 
	 curr != ((entry*) 0); 
	 curr = get_next(root,"item",curr), nitem++){
	
	/* construct default item name */
	sprintf(defname,"item%d",nitem+1);
	
	/* allocate structure to hold info for menu item */
	item = new_item(menu_info);
	
	/* get the fromlist attribute to see if we want to expand the label */
        /* if it came from a list we don't want to expand it again */
        fromlist = get_value( curr,"fromlist","false");

        /* find label, value in the next level of the tree */
        if( !strcmp( fromlist, "false" ) ){
           label = get_value(curr,"label",defname);
        }else{
           label = get_raw_value(curr,"label",defname);
        }
        item->value = get_value(curr,"value",label);
	
	/* add an item */
	narg = 0;

        /* common parameters */
        common_tags(menu,curr,args,&narg,SET_BEFORE);
        /* set no bitmap */
	XtSetArg(args[narg], XtNleftBitmap, None); narg++;
	XtSetArg(args[narg], XtNleftMargin, 50); narg++;

	XtSetArg(args[narg], XtNlabel, label); narg++;
	menuitem =  XtCreateManagedWidget(label,smeBSBObjectClass,
                          menu,args,narg);

	XtAddCallback( menuitem, XtNcallback, menubutton_callback, 
		      (XtPointer) item );

	item->label = XtNewString(label);
	item->widgetname = menuitem;
	/* callback routine needs to know the object name */
	item->object = object;
	
	/* info structure points to first item for find_item routine */
	if (nitem == 0) menu_info->firstitem = item;
	
	/* value is default; we select the button whose value matches this */
	if (nitem == 0) value = get_value(root,"value",item->value);
	
	/* check to see if this is the default item */
	/* indicate the default item with an X */
	
        if (!strcmp(value,item->value)) {
#ifndef NOMARK
 	    mark = XCreateBitmapFromData(XtDisplay(toplevel),
			     RootWindowOfScreen(XtScreen(toplevel)),
			     (char *) dot_bits,
			     dot_width, dot_height);
	    narg = 0;
	    XtSetArg(args[narg], XtNleftBitmap, mark); narg++;
#endif

	    object->value = value;
/*	    XtSetValues(menuitem,args,narg);*/
	}
    }
}

void
  menubutton_callback(widget, client_data, callData)
Widget widget;
XtPointer client_data, callData;
{
    Objdef *object;
    struct itemdef *menuitem;
    struct itemizedinfo *menu_info;
    Arg args[20];
    int narg;
    extern int quitflag;
    extern void unmark_all_items();
    
    menuitem = (struct itemdef *) client_data;
    object = (Objdef *) menuitem->object;
    menu_info = (struct itemizedinfo *) object->info;
    
    /* first unmark the previous choice, if any */
    unmark_all_items(menu_info);
    
    object->value = XtNewString((char *) menuitem->value);
    if (perform_actions(object->name,object->action,1) && !quitflag)
        quit_xtpanel(0);
    
#ifndef NOMARK
    /* indicate the current choice */
    mark = XCreateBitmapFromData(XtDisplay(toplevel),
			     RootWindowOfScreen(XtScreen(toplevel)),
			     (char *) dot_bits,
			     dot_width, dot_height);
    narg = 0;
    XtSetArg(args[narg], XtNleftBitmap, mark); narg++;
    XtSetValues(menuitem->widgetname,args,narg);
#endif
    
}

void
  unmark_all_items(menu_info)
struct itemizedinfo *menu_info;
{
    struct itemdef *item;
    Arg args[20];
    int narg;
    
    for( item=menu_info->firstitem; 
	 item != (struct itemdef*)0; 
	 item = item->next )
      {
	  narg = 0;
	  XtSetArg(args[narg], XtNleftBitmap, None); narg++;
	  XtSetValues(item->widgetname,args,narg);
      }
}

void
  menubutton_update(object,value)
Objdef* object;
char* value;
{
    struct itemizedinfo *menu_info;
    struct itemdef *item;
    Arg args[20];
    int narg;
    
    menu_info = object->info;

    for( item=menu_info->firstitem; 
	 item != (struct itemdef*)0; 
	 item = item->next )
      {
	if (!strcmp(item->value,value)) {
	  unmark_all_items(menu_info);
#ifndef NOMARK
	  narg = 0;
	  XtSetArg(args[narg], XtNleftBitmap, None); narg++;
	  XtSetValues(item->widgetname,args,narg);
	  mark = XCreateBitmapFromData(XtDisplay(toplevel),
			     RootWindowOfScreen(XtScreen(toplevel)),
			     (char *) dot_bits,
			     dot_width, dot_height);
	  narg = 0;
	  XtSetArg(args[narg], XtNleftBitmap, mark); narg++;
	  XtSetValues(item->widgetname,args,narg);
#endif
	  break;
	}
      }
}
