
/*
 * Copyright 1992 the Board of Trustees of the Leland Stanford Junior
 * University. Official permission to use this software is included in
 * the documentation. It authorizes you to use this file for any
 * non-commercial purpose, provided that this copyright notice is not
 * removed and that any modifications made to this file are commented
 * and dated in the style of the example below.
 */

/*
 *
 *  source file:   ./xtpanel/template.c
 *
 * Dave Nichols (SEP), July 17 1993
 *      wrote template.c
 */

#include <stdio.h>

#include "object.h"
#include "tree.h"
#include "builders.h"

struct tmpl_info {
	char* name;
	entry* root;
	struct tmpl_info* next;
};

static struct tmpl_info *tmpl_head = ( struct tmpl_info*)0;

void new_template( name )
char * name;
{

struct tmpl_info *curr, *newtmpl;

/* find the tail, and check for duplicate names */
if( tmpl_head != 0 ){
for( curr = tmpl_head; curr->next != 0 ; curr=curr->next ){
    if( !strcmp( curr->name, name ) ) {
	fprintf(stderr," Attempt to define two templates called %s\n",name);
	exit(-1);
    }
}
}

/* make a new one and add it to the tail */
newtmpl = (struct tmpl_info *) XtMalloc( sizeof(struct tmpl_info) );
newtmpl->name = XtNewString(name);
newtmpl->next = 0;
newtmpl->root = 0;

if( tmpl_head == 0 ){
    tmpl_head = newtmpl;
}else{
    curr->next = newtmpl;
}

}


/* find the template with the matching name */
struct tmpl_info* get_template( name )
char* name;
{
   struct tmpl_info *curr ;

   for( curr = tmpl_head; curr != 0 ; curr=curr->next ){
    if( !strcmp( curr->name, name ) ) return curr;
   }

   return 0;
}


void save_template( root )
    entry *root;
{

char* tmplname;
struct tmpl_info *newtmpl;

tmplname = get_value( root, "name", "NONAME" );

if( !strcmp( tmplname, "NONAME" ) ){
	fprintf( stderr, "A template must have a name\n");
	exit(-1);
}

newtmpl = get_template( tmplname);

if( newtmpl == 0 ){
    fprintf( stderr, "This template has not been defined: %s \n", tmplname);
    exit(-1);
}

newtmpl->root = new_entry( "template",8); 
newtmpl->root->child = root->child;

}


int is_template( name )
char* name;
{
return ( get_template( name ) != 0 );
}

/* for every string in the tree, replace oldname: by newname: */
void massage_tree( tree, oldname, newname )
entry* tree;
char* oldname;
char* newname;
{

char* mark;

  if( tree->value != 0 ){

    if( ( mark = strstr( tree->value, oldname )) != NULL ){
      char *newval = 
        (char*)XtMalloc( strlen(tree->value)+strlen(newname)-strlen(oldname)+1 );
      strncpy( newval, tree->value, mark-tree->value );
      newval[ ( mark-tree->value ) ] = '\0';
      strcat( newval, newname );
      strcat( newval, &(mark[strlen(oldname)]) );
      XtFree( tree->value );
      tree->value = newval;
    }

  }

  if( tree->next != 0 )  massage_tree( tree->next, oldname, newname );
  if( tree->child != 0 )  massage_tree( tree->child, oldname, newname );

}

void build_template(root,parent,name)
     entry *root;
     Objdef* parent;
     char* name;
{

struct tmpl_info* template;
entry* newtree;
entry *curr,*prev;
char* newname;
char *name1, *name2;


    template = get_template( name );
    if( template == 0 ){
	fprintf( stderr, "unable to find template named %s\n",name);
	fprintf( stderr, "templates must be defined before they are used\n");
	exit(-1);
    }

    newtree = copy_tree( template->root );
 
    /* find name in tree */
    newname = get_value(root,"name","");

    if ( !strcmp( newname,"") ){
	fprintf(stderr,"When using templates a name must be specified\n");
	exit(-1);
    }

    name1 = (char*)XtMalloc( strlen( newname ) +2 );
    strcpy( name1, newname ); strcat( name1, ":" );

    name2 = (char*)XtMalloc( strlen( name ) +2 );
    strcpy( name2, name ); strcat( name2, ":" );


    /* fix up the object names in the new tree */
    massage_tree( newtree, name2, name1 );

    /* remove the name entry from the first level of the new tree */
    prev=0;
    for( curr = newtree->child; curr != 0 ; curr = curr->next ){
	if( !strcmp( curr->tag, "name" )){
	    if( prev == 0 ){ newtree->child = curr->next;
	    }else{ prev->next = curr->next;}
	}else{
	  prev=curr;
	}
    }
	
    
    process_tree( newtree, parent );

}
