/*
 * Copyright 1992 the Board of Trustees of the Leland Stanford Junior
 * University. Official permission to use this software is included in
 * the documentation. It authorizes you to use this file for any
 * non-commercial purpose, provided that this copyright notice is not
 * removed and that any modifications made to this file are commented
 * and dated in the style of the example below.
 */

/*
 *
 *  source file:   ./xtpanel/window.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), June 7 1993
 *      wrote window.c
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Shell.h>
#include <X11/Xaw/Box.h>

#include "object.h"
#include "tree.h"
#include "builders.h"

void build_window(root,parent,create)
     entry *root;
     Objdef* parent;
     int create;
{
    Objdef *object;
    char *name,*label,*visible;
    Widget window,box;
    Arg args[20];
    int narg;
    char defname[12];
    static int numwin=1;
    struct compositeinfo *window_info;
    extern void window_destroy();
    extern void window_sync();

    /* create new object */
    object = new_object(parent);

    /* construct default dialog name */
    sprintf(defname,"window%d",numwin++);
    
    /* find name in tree */
    object->name = get_value(root,"name",defname);

    window_info = (struct compositeinfo*)
                   XtMalloc( sizeof( struct compositeinfo ) );

    /* create flag indicates whether a new shell window is needed */
    if (create) {
         narg = 0;
	 /* let shell resize itself if needed */
         XtSetArg(args[narg], XtNallowShellResize, TRUE ); narg++;
         window = XtCreatePopupShell(object->name,topLevelShellWidgetClass,
				parent->widgetname,args,narg);
    } else {
	 window = parent->widgetname;
    }
    /* common parameters */
    narg = 0;
    common_tags(parent->widgetname,root,args,&narg,SET_BEFORE);

    box = XtCreateManagedWidget(object->name,boxWidgetClass,
				window,args,narg);

    object->widgetname = box;
    window_info->topwidget = window;
    object->info = window_info;
    object->destroy = window_destroy;
    object->sync = window_sync;

    /* now hide object if needed */
    narg = 0;
    common_tags(window,root,args,&narg,SET_AFTER);

    /* now process the tree to build all the objects rooted in this window */
    process_tree( root, object );

    /* pop it up unless visible=false */
    visible = get_value( root, "visible", "True" );
    if( create && get_boolean(window,visible) )  XtPopup(window,XtGrabNone);

    return;
}
 
void window_sync(object,tag,value)
Objdef* object;
char* tag;
char* value;
{

    /*
     * visible must be handled here because we need to
     * call XtPopup and XtPopdown, not the manage routines
     * that are called for most objects.
     */
    if( !strcmp(tag,"visible"))
    {
	if (get_boolean(object->widgetname,value)){
        	XtPopup(XtParent(object->widgetname), XtGrabNone);
	}else{
        	XtPopdown(XtParent(object->widgetname));
	}
    } else {
    /* send everything else to update_common_tag */
        update_common_tag(object,tag,value);
    }
}

/* this routine destroys a window */
void window_destroy(object)
Objdef* object;
{
    struct compositeinfo *window_info;

    window_info = (struct compositeinfo*) object->info;

    /* destroy the outermost widget */
    XtDestroyWidget(window_info->topwidget);
}
